<?php class screen extends baseobject {
	
	protected static $fields=array(
			'name'=>validator::REQUIRED,
			'manufacturer'=>validator::ANY,
			'catalognumber'=>validator::ANY,
			'isstandard'=>validator::BOOLEAN,
	);
	
	protected static $helpTexts=array(
			'name'=>'A unique name for this screen',
			'manufacturer'=>'The manufacturer of the screen',
			'catalognumber'=>'Manufacturer\'s catalogue number',
			'isstandard'=>'Whether this is a standard screen (not an optimization)',
	);

	public static $defaultSortOrder='screen.isstandard DESC, name ASC';
	
	private static $rowLabels=array('-','A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','X','Y','Z');
	
	private static $csvMimetypes = array(
	    'text/csv',
	    'text/plain',
	    'application/csv',
	    'text/comma-separated-values',
	    'application/excel',
	    'application/vnd.ms-excel',
	    'application/vnd.msexcel',
	    'text/anytext',
	    'application/octet-stream',
	    'application/txt',
	);
	
	public static function canCreateStandardScreens(){
		$projects=session::getCreateProjects();
		if(empty($projects)){ return false; }
		$sharedProject=project::getByName(project::SHARED);
		return in_array($sharedProject['id'], $projects);
	}

	public static function canUpdate($id){
		 if(parent::canUpdate($id)){ return true; }
		 $groupid=usergroup::getByName(usergroup::TECHNICIANS);
		 return usergroup::userisingroup($groupid);
	}
	
	public static function create($request=array()){
		if(isset($request['projectid'])){
			$project=project::getById($request['projectid']);
			$request['isstandard']=false;
		} else {
			$project=project::getByName(project::SHARED);
			$request['projectid']=$project['id'];
			$request['isstandard']=true;
		}
		if(isset($request['conditions'])){
			//Installer passes in a 12x8 array of standard screen conditions with the screen
			$conditions=$request['conditions'];
		} else if(empty($_FILES)){
			//No file uploaded and no conditions passed in
			$rows=$request['rows'];
			$cols=$request['cols'];
			$conditions=static::makeDummyScreenConditions($rows, $cols, 'Unknown');
		} else {
			//File was uploaded, need to parse its conditions
			if(!isset($request['name'])){
				//Screen name is file name if none provided
				$request['name']=$_FILES['file']['name'];
			}
			$fileContent=file_get_contents($_FILES['file']['tmp_name']);
			if(false!==strpos($fileContent,'Rock Maker XML')){
				//Formulatrix Rock Maker XML file
				$conditions=static::parseRockMakerXml($fileContent, $request);
			} else if(false!==strpos($fileContent, 'CrysScreen')){
				//Tecan CrysScreen Word document (ugh)
				$conditions=static::parseTecanCrysScreenWordFile($fileContent, $request);
			} else if(false!==strpos($fileContent, 'crystaltrak ')){
				//Rigaku CrystalTrak XML file
				$conditions=static::parseCrystalTrakXml($fileContent, $request);
			} else if(in_array($_FILES['file']['type'], static::$csvMimetypes)){
				//Hopefully a CSV file. But if so, what kind?
				
				$delimiter=static::determineCsvDelimiter($fileContent);
				$fileContent=preg_split("/\r\n|\n|\r/", $fileContent);
				$firstRow=str_getcsv($fileContent[0], $delimiter);
				$thirdRow=str_getcsv($fileContent[2], $delimiter);
				
				if(2==count($thirdRow)){
					//Simple 2-column "well number, condition" CSV 
					$conditions=static::parseWellNameAndConditionCsv($fileContent);
				} else if(2<count($thirdRow)){
					//Hopefully a Mimer CSV...
					$conditions=static::parseMimerCsv($fileContent);
					if(!isset($request['name'])){
						if(empty($firstRow[0])){
							throw new BadRequestException('No name supplied, and could not find one in cell A1');
						}
						$request['name']=$firstRow[0];
					}
				} else {
					//Only one column, if it's a CSV it doesn't look like it
					throw new BadRequestException('File does not look like a screen - not enough columns');
				}
				
			} else {
				//Definitely not a recognised format, can't parse conditions - just attach it and "see file"
				$rows=$request['rows'];
				$cols=$request['cols'];
				$conditions=static::makeDummyScreenConditions($rows, $cols, 'See screen definition file');
			}
		}
		
		//Create the screen...
		$screen=parent::create($request,'screen');
		$screenId=$screen['created']['id'];
		
		//...and its conditions...
		static::createConditionsFromPlateArray($conditions, $screenId);

		//Create DB record of file and save it
		if(!empty($_FILES) && !empty($_FILES['file'])){
		    $file=file::create(array(
				'description'=>'Screen definition file',
				'parentid'=>$screenId,
    		));
		}
		return $screen;
	}

	
	public static function update($id, $request=array()){
		if(isset($request['isstandard']) && $request['isstandard']){
			$sharedProject=project::getByName(project::SHARED);
			$parameters=array(':projectid'=>$sharedProject['id'], ':id'=>$id);
			database::query(
					'UPDATE baseobject set projectid=:projectid WHERE id=:id', 
					$parameters
			);
			database::query(
					'UPDATE screen set projectid=:projectid WHERE id=:id', 
					$parameters
			);
			database::query(
					'UPDATE screencondition set projectid=:projectid WHERE screenid=:id', 
					$parameters
			);
		}
		parent::update($id,$request);
	}
	
	public static function getscreenconditions($id,$request){
		$ret=screencondition::getByProperty('screenid', $id, $request);
		$screen=static::getById($id);
		$ret['screen']=$screen;
		return $ret;
	}

	private static function parseCrystalTrakXml($fileContent){
		global $request; //ugly
		$xml=simplexml_load_string($fileContent);
		if(!$xml){ throw new BadRequestException('Detected CrystalTrak XML file but it was not valid XML'); }
		if(!empty($xml->reservoir_design) && !empty($xml->reservoir_design->format->attributes())){
			$format=$xml->reservoir_design->format->attributes();
			$wells=$xml->reservoir_design->children();
			$screenName=$xml->reservoir_design->attributes()['name'];
		} else if(!empty($xml->screen) && !Empty($xml->screen->format->attributes())){
			$format=$xml->screen->format->attributes();
			$wells=$xml->screen->children();
			$screenName=$xml->screen->attributes()['name'];
		} else {
			throw new BadRequestException('File is valid XML but does not appear to be a CrystalTrak XML file.');
		}
		if(empty($format['rows']) || empty($format['cols'])){
			throw new BadRequestException('Rows and columns not defined in screen file');
		}
		$rows=$format['rows'];
		$cols=$format['cols'];
		$request['name']=$screenName;
		$request['rows']=$rows;
		$request['cols']=$cols;
		
		$conditions=array();
		for($r=1;$r<=$rows;$r++){
			$conditions[]=array();
			for($c=1;$c<=$cols;$c++){
				$conditions[$r-1][]='';
			}
		}
		
		foreach($wells as $well){
			if($well->getName()!="well"){ 
				continue; //format and comments elements are of no interest here
			}
			$description=array();
			foreach($well->children() as $item){
				$attrs=$item->attributes();
				$desc=$attrs['class'].' '.$attrs['conc'].$attrs['units'].' '.$attrs['name'];
				if(!empty($attrs['ph'])){ $desc.=', pH '.$attrs['ph']; }
				$description[]=$desc;
			}
			$description=implode('; ', $description);

			$wellNumber=$well->attributes()['number'];
			$wellNumber=(int)$wellNumber;
			$colNumber=$wellNumber%$cols;
			if(0==$colNumber){ $colNumber=$cols; }
			$rowNumber=ceil($wellNumber/$cols);
			
			$conditions[$rowNumber-1][$colNumber-1]=$description;

		}
		return $conditions;
	}

	private static function parseRockMakerXmlToDatabase($fileContent){
		//@TODO Parse RMXML
		throw new BadRequestException('Detected Rock Maker XML file but parsing these files is not supported yet');
		$xml=simplexml_load_string($fileContent);
		if(!$xml){ throw new BadRequestException('Detected Rock Maker XML file but it was not valid XML'); }
		exit;
	}

	private static function parseMimerCsv($fileContent){
		/* Example CSV format: note that the column order may be different and row 1 may not have the numbers.
		Incomplete factorial,1,1,1,1,2,2,2,2,3,3,3,3,4,4,4,4,5,5,5,5,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
		Reagent number,PEG 8000,[PEG 8000],[PEG 8000] units,pH_PEG 8000,calcium acetate,[calcium acetate],[calcium acetate] units,pH_calcium acetate,calcium acetate,[calcium acetate],[calcium acetate] units,pH_calcium acetate,"bistris propane, pH7","[bistris propane, pH7]","[bistris propane, pH7] units","pH_bistris propane, pH7","cacodylate, pH 6.5","[cacodylate, pH 6.5]","[cacodylate, pH 6.5] units","pH_cacodylate, pH 6.5",,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
		A1,PEG 8000,8,%,,calcium acetate,0.15,M,,,,,,"bistris propane, pH7",0.1,M,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
		A2,PEG 8000,8,%,,calcium acetate,0.2,M,,,,,,"bistris propane, pH7",0.1,M,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
		A3,PEG 8000,8,%,,calcium acetate,0.25,M,,,,,,"bistris propane, pH7",0.1,M,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
		A4,PEG 8000,8,%,,,,,,calcium acetate,0.15,M,,,,,,"cacodylate, pH 6.5",0.1,M,6.5,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
		A5,PEG 8000,8,%,,,,,,calcium acetate,0.2,M,,,,,,"cacodylate, pH 6.5",0.1,M,6.5,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
		A6,PEG 8000,8,%,,,,,,calcium acetate,0.25,M,,,,,,"cacodylate, pH 6.5",0.1,M,6.5,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
		B1,PEG 8000,10,%,,calcium acetate,0.15,M,,,,,,"bistris propane, pH7",0.1,M,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,,
		*/
		if(!is_array($fileContent)){
			$fileContent=preg_split("/\r\n|\n|\r/", $fileContent);
		}
		$delimiter=static::determineCsvDelimiter($fileContent);

		$rows=0;
		$cols=0;
		/* Step 1. Parse 2nd row headers for components and their properties.
		 * We aim to create an array where key is name/role and value is an array containing the column numbers for name, amount, units and optionally pH.
		 * Later we use this to know which columns to mash together in what order.
		 * This part is made unnecessarily filthy by the wonderful world of almost-the-same-but-not-quite-just-for-the-hell-of-it screen formats. 
		 */
		$secondRow=str_getcsv($fileContent[1], $delimiter);
		$parts=array();
		//1a. First extract all the unbracketed labels that are not "pH" - these are assumed to be the chemical names/roles
		for($i=1;$i<100;$i++){ //1 because we know 0 is "reagent number" or similar
			if(!isset($secondRow[$i])){ break; }
			$columnCutoff=$i;
			$label=trim($secondRow[$i]);
			if(empty($label)){ break; }
			if(false!==strpos($label,'[') || 'pH'==$label || 0===strpos($label,'pH_')){ continue; }
			$parts[$label]=array('name'=>$i);
		}
		//1b. Now iterate through the headers again, trying to find the column numbers for each component's pH, amount, units 
		for($i=1;$i<100;$i++){ //1 because we know 0 is "reagent number" or similar
			if(!isset($secondRow[$i])){ break; }
			$label=trim($secondRow[$i]);
			if(empty($label)){ break; }
			if(false!==strpos($label,'[')){
				if('] units'==substr($label,-7)){
					$parts[substr($label,1,-7)]['unit']=$i;
				} else if(']'==substr($label,-1)){
					$parts[substr($label,1,-1)]['amount']=$i;
				} 
			} else if(false!==strpos($label,'pH_')){
				$parts[substr($label,3)]['pH']=$i;
			} else if('pH'==$label){
				//This is a pain. The label says nothing about what this is the pH *of*.
				//Work backward from this column to find something that's definitely a name/role
				for($j=0;$j<$i;$j++){
					$otherCol=$i-$j;
					$otherLabel=trim($secondRow[$otherCol]);
					if(']'==substr($otherLabel,-1)){
						$parts[substr($otherLabel,1,-1)]['pH']=$i;
						break;
					}
				}
			}
		}
		/*
		 * Step 2. Generate human-friendly representations of the conditions 
		 * Now we know which column represents what, go through each row in the file and concatenate columns in the right order.
		 */
		$fileContent=array_slice($fileContent,2); //Skip the first two rows, they are headers
		$rowLabels=implode('', static::$rowLabels);
		$wells=array();
		foreach($fileContent as $row){
            if(""==trim($row)){ continue; }
			$row=str_getcsv($row, $delimiter);
			//Row and column
			$wellLabel=trim($row[0]);
			if(empty($wellLabel)){ continue; }
			$rowLabel=substr($wellLabel,0,1);
			$rowNumber=stripos($rowLabels,$rowLabel);
			$rows=max($rows,$rowNumber);
			$columnNumber=(int)(substr($wellLabel,1));
			$cols=max($cols,$columnNumber);
			//Text description
			$labelParts=array();
			foreach($parts as $k=>$v){
				if(!isset($row[$v['amount']]) || !isset($row[$v['unit']]) || !isset($row[$v['name']])){
					continue;
				}
				$str=trim($row[$v['amount']].$row[$v['unit']].' '.$row[$v['name']]);
				if(!empty($str)){
					if(isset($v['pH']) && !empty($row[$v['pH']]) && false==strpos($str,'pH')){
						$str.=', pH '.$row[$v['pH']];
					}
					$labelParts[]=$str;
				}
			}
			$label=implode('; ',$labelParts);
			$well=array(
					'description'=>$label,
					'row'=>$rowNumber,
					'col'=>$columnNumber
			);
			$wells[]=$well;
		}
		
		$conditions=array();
		for($r=1;$r<=$rows;$r++){
			$conditions[]=array();
			for($c=1;$c<=$cols;$c++){
				$conditions[$r-1][]='';
			}
		}
		
		foreach($wells as $w){
			$r=$w['row'];
			$c=$w['col'];
			$conditions[$r-1][$c-1]=$w['description'];
		}
		
		return $conditions;
	}

	/**
	 * Parses a Tecan CrysScreen Word document's well conditions.
	 * Assumes that the document has a table with B:(buffer) P:(precipitant) S:(salt) in each cell, and that the wells appear
	 * in the document in A1-A12, B1-B12, etc., order. 
	 * @param string $fileContent The contents of the file
	 * @return array A 2D array of screen conditions
	 * @throws BadRequestException if file does not contain exactly 96 conditions
	 */
	private static function parseTecanCrysScreenWordFile($fileContent){
	
		if(!is_array($fileContent)){
			$fileContent=preg_split("/\r\n|\n|\r/", $fileContent);
		}
		$delimiter=',';

		$conditions=array(
				array('', '', '', '', '', '', '', '', '', '', '', ''),
				array('', '', '', '', '', '', '', '', '', '', '', ''),
				array('', '', '', '', '', '', '', '', '', '', '', ''),
				array('', '', '', '', '', '', '', '', '', '', '', ''),
				array('', '', '', '', '', '', '', '', '', '', '', ''),
				array('', '', '', '', '', '', '', '', '', '', '', ''),
				array('', '', '', '', '', '', '', '', '', '', '', ''),
				array('', '', '', '', '', '', '', '', '', '', '', ''),
		);
		
		$wholeScreen='';
		foreach($fileContent as $row){
			$row=trim($row);
			if(!preg_match("/^[BPSA]:/", $row)){ continue; }
			$wholeScreen.=$row.' ';
		}
		$wholeScreen=substr($wholeScreen, 2);
		$wholeScreen=preg_replace('/[^\x20-\x7E]*/', '', $wholeScreen);
		$wholeScreen=preg_replace('/ pH:/', ', pH ', $wholeScreen);
		$parts=explode('B:', $wholeScreen); //Buffer appears first in the description
		if(count($parts)!=96){
			throw new BadRequestException('Could not parse CrysScreen file, found '.count($conditions).' conditions instead of 96');
		}

		for($i=0;$i<count($parts); $i++){
			$condition=preg_replace('/\s[BPSA]:\s/', '; ', trim($parts[$i]));
			$conditionNumber=$i;
			$row=floor($conditionNumber/12);
			$col=$conditionNumber%12;
			$condition=trim($condition);
			if(''==$condition){ throw new BadRequestException('Condition '.(1+$i).' was empty'); }
			$conditions[$row][$col]=$condition;
		}
		
		return $conditions;
	}

	/**
	 * Parses a CSV file with just well and condition into the database
	 * A1,Condition
	 * A01,Condition
	 * 1,Condition - assumes numbering A1-A12,B1-B12, etc.
	 * @param string $fileContent
	 * @throws BadRequestException if file does not contain exactly 96 conditions or any well ID is outside 12x8
	 */
	public static function parseWellNameAndConditionCsv($fileContent){
		//Public because installer uses it.
	
		//We'll populate this from the CSV
		$conditions=array(
				array('', '', '', '', '', '', '', '', '', '', '', ''),
				array('', '', '', '', '', '', '', '', '', '', '', ''),
				array('', '', '', '', '', '', '', '', '', '', '', ''),
				array('', '', '', '', '', '', '', '', '', '', '', ''),
				array('', '', '', '', '', '', '', '', '', '', '', ''),
				array('', '', '', '', '', '', '', '', '', '', '', ''),
				array('', '', '', '', '', '', '', '', '', '', '', ''),
				array('', '', '', '', '', '', '', '', '', '', '', ''),
		);
		
		if(!is_array($fileContent)){
			$fileContent=preg_split("/\r\n|\n|\r/", $fileContent);
		}
		
		$delimiter=',';
		$firstRow=str_getcsv($fileContent[0], $delimiter);
		if(2>count($firstRow)){
			$delimiter=';';
			$firstRow=str_getcsv($fileContent[0], $delimiter);
			if(2>count($firstRow)){
				throw new BadRequestException('Could not parse CSV file, tried comma and semicolon as delimiters');
			} else if(2<count($firstRow)){
				throw new BadRequestException('Expected only two columns: Well identifier, and condition');
			}
		}
		foreach($fileContent as $row){
			$row=str_getcsv($row, $delimiter);
			if(empty($row) || empty($row[0])){ continue; } //blank line at end of file
			if('Well'==$row[0]){ continue; } //header row
				
			//Row and column
			$colNumber=0;
			$rowNumber=0;
			$wellLabel=trim($row[0]);
			$condition=trim($row[1]);
			if(preg_match('/^[A-Z]\d\d?$/',$wellLabel)){
				$rowLabel=substr($wellLabel, 0, 1);
				$colNumber=(int)(substr($wellLabel,1));
				$rowNumber=array_search($rowLabel, static::$rowLabels);
				$wellNumber=(12*($rowNumber-1))+$colNumber;
			} else {
				$wellNumber=(int)$wellLabel;
				$colNumber=$wellNumber%12;
				if(0==$colNumber){ $colNumber=12; }
				$rowNumber=ceil($wellNumber/12);
			}
			if($rowNumber<1 || $rowNumber>8 || $colNumber<1 || $colNumber>12){
				throw new BadRequestException('Well identifier '.$wellLabel.' not recognised or out of bounds');
			}
			$conditions[$rowNumber-1][$colNumber-1]=$condition;
		}
	
		return $conditions;
	}

	private static function makeDummyScreenConditions($rows, $cols, $dummyText){
		$conditions=array();
		$separator='|---|';
		$rowText=str_repeat($dummyText.$separator, $cols-1) . $dummyText;
		for($r=1;$r<=$rows;$r++){
			$conditions[]=explode($separator, $rowText);
		}
		return $conditions;
	}
	
	
	/**
	 * Takes an (8x12) array of screen conditions and a screen ID, and sets the screen conditions against the screen.
	 * @param array $plateArray
	 * @param int|string $screenId
	 * @throws ServerException
	 */
	public static function createConditionsFromPlateArray($plateArray, $screenId){
		//public because installer uses it
		$screen=screen::getById($screenId);
		if(!$screen){ throw new ServerException('screen::createConditionsFromPlateArray got bad screen ID'); }
		if(!is_array($plateArray)){ throw new ServerException('screen::createConditionsFromPlateArray got non-array'); }
		$projectId=$screen['projectid'];
		$rowCount=count($plateArray);
		$colCount=count($plateArray[0]);
		for($r=1;$r<=$rowCount;$r++){
			if($colCount!=count($plateArray[$r-1])){
				throw new ServerException('screen::createConditionsFromPlateArray found inconsistent column count in plate array');
			}
			for($c=1;$c<=$colCount;$c++){
				$description=$plateArray[$r-1][$c-1];
				if(empty($description)){
					throw new BadRequestException('No condition found for well '.self::$rowLabels[$r].$c.'. Are plate and screen the same size?');
				}
				$wellNumber=($colCount*($r-1))+$c;
				$condition=array(
						'name'=>'screen'.$screenId.'_well'.str_pad($wellNumber, 3, '0', STR_PAD_LEFT),
						'description'=>$description,
						'wellnumber'=>$wellNumber,
						'screenid'=>$screenId,
						'projectid'=>$projectId,
						'row'=>$r,
						'col'=>$c
				);
				screencondition::create($condition);
			}
		}
	}
	
	private static function determineCsvDelimiter($fileContent){
		if(!is_array($fileContent)){
			$fileContent=preg_split("/\r\n|\n|\r/", $fileContent);
		}
		if(3>count($fileContent)){
			throw new BadRequestException('Supplied CSV is too short');
		}
		$delimiters=array(",", ";", "\t");
		foreach($delimiters as $d){
			$firstRow=str_getcsv($fileContent[0], $d);
			$thirdRow=str_getcsv($fileContent[2], $d);
			if(count($firstRow)==count($thirdRow)){
				return $d;
			}
		}
		throw new BadRequestException('Could not determine delimiter of CSV file');	
	}
	
}


?>