#!/bin/bash

echo "Making storage directory /icebearstore/"
mkdir /icebearstore/
chown www-data /icebearstore/

echo "Making log directory /var/log/icebear/"
mkdir /var/log/icebear/
chown www-data /var/log/icebear/

echo "Setting up temporary cron job for later install tasks"
HERE="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
croncmd="$HERE/rootinstalltasks.sh"
cronjob="* * * * * $croncmd"
( crontab -l | grep -v -F "$croncmd" ; echo "$cronjob" ) | crontab -

echo "Renaming default Apache index file"
mv /var/www/html/index.html /var/www/html/index_old.html

echo "Configuring Apache - enable mod_rewrite"
a2enmod -q rewrite

echo "Configuring Apache - set AllowOverride"
#Enable .htaccess - 
# - read in from /etc/apache2/apache2.conf
# - set first instance of AllowOverride after /var/www/ from None to All
# - write out to /etc/apache2/tmp
# - rename /etc/apache2/tmp to /etc/apache2/apache2.conf
awk 'f==1 && /AllowOverride None/{$0="AllowOverride All";f=0}/Directory \/var\/www/{f=1}1' /etc/apache2/apache2.conf > /etc/apache2/tmp && mv /etc/apache2/tmp /etc/apache2/apache2.conf

echo "Configuring PHP - installing Sybase module to communicate with Formulatrix SQL Server database"
apt-get --assume-yes install php7.0-sybase

echo "Configuring PHP - modifying php.ini files to enable MSSQL secure connection"
echo " " >> /etc/php/7.0/apache2/php.ini
echo "#Added by Icebear installer, allows connection to MSSQL without pre-existing DSN" >> /etc/php/7.0/apache2/php.ini
echo "mssql.secure_connection = on" >> /etc/php/7.0/apache2/php.ini
echo " " >> /etc/php/7.0/cli/php.ini
echo "#Added by Icebear installer, allows connection to MSSQL without pre-existing DSN" >> /etc/php/7.0/cli/php.ini
echo "mssql.secure_connection = on" >> /etc/php/7.0/cli/php.ini

echo "Restarting Apache to refresh Apache and PHP config"
systemctl restart apache2

echo "Installing cifs-utils package to enable mounting Formulatrix image stores"
apt-get --assume-yes install cifs-utils

echo "Setting up a firewall"
ufw default deny incoming
ufw default allow outgoing
ufw allow http
ufw allow https
ufw allow ssh
#ufw enable
systemctl enable ufw.service

touch /var/www/html/install/preinstalldone

echo " **********************************************************************************"
echo " *                                                                                *"
echo " * Icebear pre-install script complete.                                           *"
echo " *                                                                                *"
echo " * FIREWALL                                                                       *"
echo " *                                                                                *"
echo " * This install script has activated a firewall (ufw) and created some basic      *"
echo " * rules, but YOU are responsible for ensuring the security of this machine! Ask  *"
echo " * your IT department for advice.                                                 *"
echo " *                                                                                *"
echo " * SECURE HTTP (HTTPS)                                                            *"
echo " *                                                                                *"
echo " * The Apache web server on this machine should be configured to encrypt traffic  *"
echo " * via secure HTTP (URLs begin with https:// instead of http://). This is NOT     *"
echo " * the default, meaning that usernames, passwords, and scientific data travel     *"
echo " * UNENCRYPTED between server and browser. You really should configure Apache to  *"
echo " * use secure HTTP. You will almost certainly need to get a certificate from your *"
echo " * IT department for this. Ask them for advice.                                   *"
echo " *                                                                                *"
echo " * IMPORTANT: STORAGE AND BACKUP DIRECTORIES                                      *"
echo " *                                                                                *"
echo " * By default, Icebear stores file uploads and imported images at /icebearstore   *"
echo " * and NO BACKUP is configured. If you choose to put the Icebear store in a       *"
echo " * different location, you must create that directory now and ensure that it is   *"
echo " * WRITABLE by the Apache user (probably www-data).                               *"
echo " *                                                                                *"
echo " * Unless you have another backup mechanism in place, you really should have      *"
echo " * Icebear take backups. The backups should AT LEAST be on a different disk, but  *"
echo " * should really be as far from this machine as is practical. As with the storage *"
echo " * directory, you need to mount the backup directory now. The backup directory    *"
echo " * must be READABLE to the Apache user (probably www-data).                       *"
echo " *                                                                                *"
echo " * However you back up Icebear, you should check regularly that backups are being *"
echo " * taken AND that you can restore from them!                                      *"
echo " *                                                                                *"
echo " * You will configure the storage and backup directories later in the install     *"
echo " * process, but the directories must exist before you continue.                   *"
echo " *                                                                                *"
echo " * Continue in your web browser by going to the index page on this server,        *"
echo " * e.g., http://this-machine.myuni.edu                                            *"
echo " *                                                                                *"
echo " **********************************************************************************"

####################
# End of pre-install
####################

#For reference only
#To add it to the crontab, with no duplication:
#( crontab -l | grep -v -F "$croncmd" ; echo "$cronjob" ) | crontab -
#To remove it from the crontab:
#( crontab -l | grep -v -F "$croncmd" ) | crontab -
